from __future__ import absolute_import

import hashlib
import json
import os
import os.path
import xml.dom.minidom
from collections import OrderedDict
from contextlib import closing

import requests

import pythesint.utils as utils
from pythesint.json_vocabulary import JSONVocabulary
from pythesint.pathsolver import DATA_HOME

class MMDVocabulary(JSONVocabulary):
    base_url = 'https://raw.githubusercontent.com/metno/mmd/master/thesauri/mmd-vocabulary.xml'
    base_file = os.path.join(DATA_HOME, 'pythesint', 'mmd-vocabulary.xml')

    @staticmethod
    def get_subnode_data(node, subnode_name):
        """Get the data contained in a subnode. Return an empty string
        if the subnode does not contain any data.
        """
        try:
            return node.getElementsByTagName(subnode_name)[0].childNodes[0].data.strip()
        except IndexError:
            return ''

    def get_element_by_label(self, parent, node_type, label):
        """Returns the collection which has the given label"""
        for node in parent.getElementsByTagName(node_type):
            if self.get_subnode_data(node, 'skos:prefLabel') == label:
                return node
        return None

    @staticmethod
    def _get_local_file_github_blob_sha(file_path):
        """Get the local file's blob SHA hash as used by github"""
        if os.path.isfile(file_path):
            with open(file_path, 'rb') as f_h:
                contents = f_h.read()
            header_string = f"blob {len(contents)}\0"
            blob = bytes(header_string, encoding='utf-8') + contents
            return hashlib.sha1(blob).hexdigest()
        return None

    def _get_remote_file_github_blob_sha(self, version):
        """Get the remote file's blob sha hash from github"""
        endpoint = f"https://api.github.com/repos/metno/mmd/git/trees/{version}:thesauri"
        result = json.loads(requests.get(endpoint).text)
        for file_info in result['tree']:
            if file_info['path'] == 'mmd-vocabulary.xml':
                return file_info['sha']
        return None

    def _download_data_file(self, version=None):
        """Download the file containing all the vocabularies
        definitions if it is not already present
        """
        if version:
            url = utils.set_github_version(self.base_url, version)
        else:
            url = self.base_url

        # by comparing the hashes, we can check if we need to
        # download the file or not
        local_sha = self._get_local_file_github_blob_sha(self.base_file)
        remote_sha = self._get_remote_file_github_blob_sha(version)

        if (not os.path.isfile(self.base_file) or local_sha != remote_sha):
            with closing(requests.get(url, stream=True)) as response:
                with open(self.base_file, 'wb') as f_h:
                    f_h.write(response.content)

    def _fetch_online_data(self, version=None):
        """Download, if necessary, the file containing all the MMD
        vocabularies definitions. Then extract the requested collection
        and write it to a JSON file like any JSON vocabulary.
        """
        if not version:
            version = 'master'

        self._download_data_file(version=version)

        with open(self.base_file, 'r', encoding='utf-8') as f_h:
            dom = xml.dom.minidom.parse(f_h)

        document = dom.documentElement

        collection = self.get_element_by_label(document, 'skos:Collection', self.collection_label)

        if collection is None:
            raise ValueError(f"'{self.collection_label}' is not a valid collection label")

        definition = self.get_subnode_data(collection, 'skos:definition')
        details = OrderedDict([
            ('aboutCollection', collection.getAttribute('rdf:about')),
            ('prefLabelCollection', self.collection_label),
            ('definitionCollection', definition)])

        if version:
            details['version'] = version

        mmd_list = [details]
        for cnode in collection.getElementsByTagName('skos:member'):
            try:
                # some concepts are directly defined in the collection
                # members...
                concept = cnode.getElementsByTagName('skos:Concept')[0]
            except (AttributeError, IndexError):
                # ...while others are defined outside of the collection
                # and need to be retrieved using their resource URL
                resource_name = cnode.getAttribute('rdf:resource')

                concept = None
                for concept_element in document.getElementsByTagName('skos:Concept'):
                    if concept_element.getAttribute('rdf:about') == resource_name:
                        concept = concept_element
                        break

                if not concept:
                    continue

            label = self.get_subnode_data(concept, 'skos:prefLabel')
            definition = self.get_subnode_data(concept, 'skos:definition')
            access_constraint = OrderedDict([
                ('prefLabel', label),
                ('definition', definition)
            ])
            mmd_list.append(access_constraint)

        return mmd_list



'''
Created on Feb 25, 2016

@author: alevin
'''
from __future__ import absolute_import

from collections import OrderedDict

import unittest
from mock.mock import MagicMock

from pythesint.vocabulary import Vocabulary


class VocabularyTest(unittest.TestCase):

    def setUp(self):
        self.cat = OrderedDict([('Category', 'Animal'), ('Type', 'Cat'), ('Name', '')])
        self.cat2 = OrderedDict([('Category', 'Animal'), ('Type', 'Cat'), ('Name', 'The cat')])
        self.dog = OrderedDict([('Category', 'Animal'), ('Type', 'Dog'), ('Name', '')])
        self.mouse = OrderedDict([('Category', 'Animal'), ('Type', 'Mouse'), ('Name', '')])
        self.house = OrderedDict([('Category', 'Construction'), ('Type', 'House'), ('Name', '')])
        self.animal = OrderedDict([('Category', 'Animal'), ('Type', ''), ('Name', '')])
        # The order of this is list important for which value that is returned
        # in case of multiple values of "best match"
        self.test_list = [self.cat, self.cat2, self.animal, self.dog, self.mouse, self.house]

    def test_find_keyword_get_list_not_implemented(self):
        vocab = Vocabulary('VOCAB MOCK')
        with self.assertRaises(NotImplementedError):
            vocab.find_keyword('an item')

    def test_find_keyword_not_found(self):
        vocab = Vocabulary('VOCAB MOCK')
        vocab.get_list = MagicMock(return_value=self.test_list)
        with self.assertRaises(IndexError):
            vocab.find_keyword('Horse')

    def test_find_keyword(self):
        vocab = Vocabulary('VOCAB MOCK')
        vocab.get_list = MagicMock(return_value=self.test_list)
        self.assertEqual(vocab.find_keyword('dog'), self.dog)
        self.assertEqual(vocab.find_keyword('Animal'), self.animal)

    def test_search_keyword(self):
        vocab = Vocabulary('VOCAB MOCK')
        vocab.get_list = MagicMock(return_value=self.test_list)
        self.assertEqual(vocab.search('dog'), [self.dog])
        self.assertEqual(vocab.search('Animal'), [self.cat,
                                                  self.cat2,
                                                  self.animal,
                                                  self.dog,
                                                  self.mouse])

    def test_no_duplicate_in_search(self):
        vocab = Vocabulary('VOCAB MOCK')
        vocab.get_list = MagicMock(return_value=self.test_list)
        self.assertEqual(vocab.search('Cat'), [self.cat, self.cat2])

    def test_no_empty_dict_in_sort_output(self):
        vocabulary = Vocabulary('test_vocabulary', categories=['foo', 'bar'])
        entry = {'foo': 'a', 'bar': 'b'}
        vocabulary_list = [{'Revision': '2016-01-08 13:40:40'}, entry]
        self.assertListEqual(vocabulary.sort_list(vocabulary_list), [OrderedDict(entry)])

    def test_sort_list(self):
        vocabulary = Vocabulary('test_vocabulary', categories=['Category', 'Type', 'Name'])
        self.assertListEqual(
            vocabulary.sort_list(self.test_list),
            [
                OrderedDict([('Category', 'Animal'), ('Type', 'Cat'), ('Name', '')]),
                OrderedDict([('Category', 'Animal'), ('Type', 'Cat'), ('Name', 'The cat')]),
                OrderedDict([('Category', 'Animal'), ('Type', ''), ('Name', '')]),
                OrderedDict([('Category', 'Animal'), ('Type', 'Dog'), ('Name', '')]),
                OrderedDict([('Category', 'Animal'), ('Type', 'Mouse'), ('Name', '')]),
                OrderedDict([('Category', 'Construction'), ('Type', 'House'), ('Name', '')]),
            ])

    def test_sort_list_aliases(self):
        vocabulary = Vocabulary('test_vocabulary', categories={
            'Category': 'class',
            'Type': 'kind',
            'Name': ''
        })
        self.assertListEqual(
            vocabulary.sort_list(self.test_list),
            [
                OrderedDict([('class', 'Animal'), ('kind', 'Cat'), ('Name', '')]),
                OrderedDict([('class', 'Animal'), ('kind', 'Cat'), ('Name', 'The cat')]),
                OrderedDict([('class', 'Animal'), ('kind', ''), ('Name', '')]),
                OrderedDict([('class', 'Animal'), ('kind', 'Dog'), ('Name', '')]),
                OrderedDict([('class', 'Animal'), ('kind', 'Mouse'), ('Name', '')]),
                OrderedDict([('class', 'Construction'), ('kind', 'House'), ('Name', '')]),
            ])


if __name__ == "__main__":
    # import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



"""Utilities"""

import re

def set_github_version(url, version):
    """Returns a modified `url` using `version`. Only works for raw
    content URLs from github
    """
    return re.sub(
        r'(https://raw.githubusercontent.com/([^/]+/){2})[^/]+/',
        r'\g<1>{}/'.format(version),
        url)



import unittest

import mock.mock as mock
import requests

from pythesint.wkv_vocabulary import WKVVocabulary


class WKVVocabularyTest(unittest.TestCase):
    def test_exception_on_unavailable_remote_file(self):
        voc = WKVVocabulary(name='test_voc', url='https://sdfghdfghd.nersc.no')
        with self.assertRaises(requests.RequestException):
            voc._fetch_online_data()

    def test_fetch_version(self):
        """Test that the specified version is fetched"""
        voc = WKVVocabulary(
            name='test_voc',
            url='https://raw.githubusercontent.com/nansencenter/nersc-vocabularies/master/nansat_wkv.yml')
        # We use the exception as side effect to skip the part of the
        # method we are not testing.
        # TODO: split _fetch_online_data() into several methods instead
        with mock.patch('requests.get') as mock_get:
            mock_get.return_value.text = '[]'
            self.assertListEqual(voc._fetch_online_data(), [])
            mock_get.assert_called_with(
                'https://raw.githubusercontent.com/nansencenter/nersc-vocabularies/master/'
                'nansat_wkv.yml')

            self.assertListEqual(voc._fetch_online_data(version=None), [])
            mock_get.assert_called_with(
                'https://raw.githubusercontent.com/nansencenter/nersc-vocabularies/master/'
                'nansat_wkv.yml')

            self.assertListEqual(voc._fetch_online_data(version=''), [])
            mock_get.assert_called_with(
                'https://raw.githubusercontent.com/nansencenter/nersc-vocabularies/master/'
                'nansat_wkv.yml')

            self.assertListEqual(
                voc._fetch_online_data(version='91912bb'),
                [{'version': '91912bb'}])
            mock_get.assert_called_with(
                'https://raw.githubusercontent.com/nansencenter/nersc-vocabularies/91912bb/'
                'nansat_wkv.yml')



import unittest

import mock.mock as mock
import requests

from pythesint.cf_vocabulary import CFVocabulary

class CFVocabularyTest(unittest.TestCase):
    def test_exception_on_unavailable_remote_file(self):
        voc = CFVocabulary(name='test_voc', url='https://sdfghdfghd.nersc.no')
        with self.assertRaises(requests.RequestException):
            voc._fetch_online_data()

    def test_fetch_version(self):
        """Test that the specified version is fetched"""
        voc = CFVocabulary(name='test_voc', url='https://sdfghdfghd.nersc.no')
        # We use the exception as side effect to skip the part of the
        # method we are not testing.
        # TODO: split _fetch_online_data() into several methods instead
        with mock.patch('requests.get', side_effect=requests.RequestException) as mock_get:
            with self.assertRaises(requests.RequestException):
                voc._fetch_online_data()
            mock_get.assert_called_with('https://sdfghdfghd.nersc.no', params={})

            with self.assertRaises(requests.RequestException):
                voc._fetch_online_data(version=None)
            mock_get.assert_called_with('https://sdfghdfghd.nersc.no', params={})

            with self.assertRaises(requests.RequestException):
                voc._fetch_online_data(version='')
            mock_get.assert_called_with('https://sdfghdfghd.nersc.no', params={})

            with self.assertRaises(requests.RequestException):
                voc._fetch_online_data(version='9.1.5')
            mock_get.assert_called_with(
                'https://sdfghdfghd.nersc.no', params={'version': '9.1.5'})



from __future__ import absolute_import

from xml.dom.minidom import parseString
import requests
from pythesint.json_vocabulary import JSONVocabulary


class CFVocabulary(JSONVocabulary):
    def _fetch_online_data(self, version=None):
        if version:
            params = {'version': version}
        else:
            params = {}

        try:
            r = requests.get(self.url, params=params)
        except requests.RequestException:
            print("Could not get the vocabulary file at '{}'".format(self.url))
            raise
        dom = parseString(r.text.encode('utf-8').strip())
        # should only contain the standard_name_table:
        node = dom.childNodes[0]

        details = {}
        metadata_details = node.getElementsByTagName('owl:Ontology')[0]
        for detail in metadata_details.childNodes:
            # This does not work in python2.7 because type(detail)=instance
            #if type(detail)==xml.dom.minidom.Element:
            try:
                details[detail.nodeName] = detail.childNodes[0].data
            except (AttributeError, IndexError):
                continue

        cf_list = [details]
        for cnode in node.getElementsByTagName('Standard_Name')[0].childNodes:
            #if type(cnode)==xml.dom.minidom.Element:
            try:
                entry = cnode.getElementsByTagName('Standard_Name')[0]
            except (AttributeError, IndexError):
                continue
            else:
                standard_name = entry.getAttribute('rdf:about')
                units = ''
                if entry.getElementsByTagName('canonical_units'):
                    units = entry.getElementsByTagName(
                        'canonical_units')[0].childNodes[0].nodeValue
                if entry.getElementsByTagName('skos:definition'):
                    definition = entry.getElementsByTagName(
                            'skos:definition')[0].childNodes[0].nodeValue
                stdname = {
                    'standard_name': standard_name,
                    'canonical_units': units,
                    'definition': definition
                }
                cf_list.append(stdname)
        return cf_list




from __future__ import absolute_import
from os import environ

try:
    from xdg import (XDG_DATA_HOME)
except ImportError:
    XDG_DATA_HOME = '.'

import os

def _get_data_home():
    if os.name == 'nt':
        if "XDG_DATA_HOME" in environ:
            return environ.get("XDG_DATA_HOME")
        else:
            return environ.get("LOCALAPPDATA")
    else:
        return XDG_DATA_HOME

DATA_HOME = _get_data_home()



from __future__ import absolute_import

from pkg_resources import resource_string

import yaml

def update_all_vocabularies(versions=None):
    ''' Update all vocabularies '''
    if not versions:
            versions = {}
    for name in vocabularies.keys():
        version = versions.get(name)
        if not version:
            try:
                version = vocabularies[name].version
            except AttributeError:
                version = None
        vocabularies[name].update(version=version)

def _process_config():
    ''' Load info about all vocabularies from config and add to module '''
    # http://stackoverflow.com/questions/1621350/dynamically-adding-functions-to-a-python-module
    # http://stackoverflow.com/questions/4821104/python-dynamic-instantiation-from-string-name-of-a-class-in-dynamically-imported

    # load config file
    config = yaml.load(resource_string(__name__, 'pythesintrc.yaml'), Loader=yaml.FullLoader)
    current_module = __import__(__name__)

    # add get_ and update_ functions
    for cnf in config:
        voc_module = __import__('pythesint.' + cnf['module'],
                                fromlist=[__name__])
        voc_class = getattr(voc_module, cnf['class'])
        vocabulary = voc_class(cnf['name'], **cnf['kwargs'])
        vocabularies[cnf['name']] = vocabulary

        setattr(current_module, 'get_'+cnf['name'], vocabulary.find_keyword)
        setattr(current_module,
                'get_'+cnf['name']+'_list',
                vocabulary.get_list)
        setattr(current_module,
                'search_'+cnf['name']+'_list',
                vocabulary.search)
        setattr(current_module, 'update_'+cnf['name'], vocabulary.update)

vocabularies = {}
_process_config()



'''
Created on Feb 25, 2016

@author: alevin
'''
from __future__ import absolute_import

import unittest
import mock.mock as mock
try:
    from StringIO import StringIO  # python 2
    StringIO.__enter__ = lambda *args: args[0]
    StringIO.__exit__ = lambda *args: args[0]
except ImportError:
    from io import StringIO  # python 3


from pythesint.json_vocabulary import JSONVocabulary


class JSONVocabularyTest(unittest.TestCase):

    def setUp(self):
        patch_fetch = mock.patch.object(JSONVocabulary, '_fetch_online_data', return_value=[])
        self.mock_fetch = patch_fetch.start()
        self.addCleanup(patch_fetch.stop)

    def test_update_no_version(self):
        """no version is provided"""
        vocabulary = JSONVocabulary(name='test')
        # avoid writing to a real file
        with mock.patch('pythesint.json_vocabulary.open', return_value=StringIO()):
            vocabulary.update()
            vocabulary._fetch_online_data.assert_called_with(version=None)

    def test_update_with_version(self):
        """a version is provided as argument"""
        vocabulary = JSONVocabulary(name='test')
        # avoid writing to a real file
        with mock.patch('pythesint.json_vocabulary.open', return_value=StringIO()):
            vocabulary.update(version='9.1.5')
            vocabulary._fetch_online_data.assert_called_with(version='9.1.5')

    def test_update_version_in_config(self):
        """a version is provided in the configuration"""
        vocabulary = JSONVocabulary(name='test', version='9.1.4')
        # avoid writing to a real file
        with mock.patch('pythesint.json_vocabulary.open', return_value=StringIO()):
            vocabulary.update()
            vocabulary._fetch_online_data.assert_called_with(version='9.1.4')

    def test_update_override_config_version(self):
        """"""
        vocabulary = JSONVocabulary(name='test', version='9.1.4')
        # avoid writing to a real file
        with mock.patch('pythesint.json_vocabulary.open', return_value=StringIO()):
            vocabulary.update(version='9.1.5')
            vocabulary._fetch_online_data.assert_called_with(version='9.1.5')



from collections import OrderedDict

class Vocabulary(object):
    def __init__(self, name, **kwargs):
        self.name = name
        for key in kwargs:
            setattr(self, key, kwargs[key])

    def find_keyword(self, item):
        ''' Return the dictionary containing the given item in the provided list.
        The function returns only the lowest level match, i.e., the
        dictionary where the subgroups of the matching group are empty or the only
        match, if there is just one.

        Examples:

        1) Searching "Active Remote Sensing" in the instruments list will return
            {
                'Category': 'Earth Remote Sensing Instruments',
                'Class': 'Active Remote Sensing',
                'Type': '',
                'Subtype': '',
                'Short_Name': '',
                'Long_Name': ''
            }

        2) Searching "ASAR" in the instruments list will return
            {
                'Category': 'Earth Remote Sensing Instruments',
                'Class': 'Active Remote Sensing',
                'Type': 'Imaging Radars',
                'Subtype': '',
                'Short_Name': 'ASAR',
                'Long_Name': 'Advanced Synthetic Aperature Radar'
            }

        3) Searching "surface_backwards_scattering_coefficient_of_radar_wave" will
        return

        DOCTESTS:

        '''
        best_match = None
        best_match_empty_cells = 0
        for d in self.get_list():
            match = None
            match_empty_cells = 0
            for _, key in enumerate(d.keys()):
                if d[key].upper() == item.upper():
                    match = d
                # sum up number of empty cells after matched cell
                if match == d and len(d[key]) == 0:
                    match_empty_cells += 1
            # if match is found in this row, and
            #     if the best match is not found yet, or
            #     if the best match has less empty cells after matched cell
            # i.e., if we search for the word 'animal', then the dictionary
            # {'animal', ''} is a better match than {'animal', 'cat'}
            if (match is not None and
                (best_match is None or
                 match_empty_cells > best_match_empty_cells)):
                best_match = match
                best_match_empty_cells = match_empty_cells

        if best_match is None:
            raise IndexError('%s is not found in %s!' % (item, self.name))

        return best_match

    def search(self, keyword):
        ''' Search for keyword in a list and return all matches '''
        matches = []
        for d in self.get_list():
            for key in d.keys():
                if keyword.upper() in d[key].upper() and not d in matches:
                    matches.append(d)

        return matches

    def update(self, version=None):
        pass

    def sort_list(self, list):
        retlist = []
        for dd in list:
            line_kw = OrderedDict()
            has_aliases = isinstance(self.categories, dict)
            try:
                for key in self.categories:
                    if has_aliases:
                        # self.categories is a dict defining aliases
                        alias = self.categories.get(key)
                        # if an empty alias is defined, use the original category name
                        field_name = alias if alias else key
                    else:
                        # no alias is defined, just use the category name
                        field_name = key
                    line_kw[field_name] = dd[key]
            except KeyError:
                continue
            retlist.append(line_kw)
        return retlist

    def get_list(self):
        raise NotImplementedError




from __future__ import absolute_import

import os
import json

from pythesint.vocabulary import Vocabulary
from pythesint.pathsolver import DATA_HOME

class JSONVocabulary(Vocabulary):

    def _fetch_online_data(self, version=None):
        raise NotImplementedError

    def get_relative_path(self):
        return os.path.join('pythesint', 'json', '%s_list.json' % self.name.lower())

    def get_list(self):
        ''' Read list from JSON '''
        if not os.path.exists(self.get_filepath()):
            self.update()
        with open(self.get_filepath(), 'r') as opened_file:
            result = json.load(opened_file)
        return self.sort_list(result)

    def update(self, version=None):
        ''' Write vocabulary to a JSON file '''
        if not version:
            try:
                version = self.version
            except AttributeError:
                version = None
        print('Downloading and writing json file for %s' % self.name)
        json_path = os.path.split(self.get_filepath())[0]
        print(json_path)
        if not os.path.exists(json_path):
            os.makedirs(json_path)
        with open(self.get_filepath(), 'w') as out:
            json.dump(self._fetch_online_data(version=version), out, indent=4)

    def get_filepath(self):
        return os.path.join(DATA_HOME, self.get_relative_path())



from __future__ import absolute_import
from __future__ import print_function

import requests
import collections
import unittest
import os
import shutil
from pkg_resources import resource_filename, resource_string

import pythesint as pti
from mock.mock import MagicMock, patch
from pythesint.pathsolver import DATA_HOME


class PythesintTest(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        # Delete stored json data to work with a clean slate
        try:
            shutil.rmtree(os.path.join(DATA_HOME, 'pythesint', 'json'))
        except OSError:
            pass

    def test_update_all(self):
        pti.update_all_vocabularies()
        for _, vocab in pti.pythesint.vocabularies.items():
            self.assertIsInstance(vocab.get_list(), list)

    def test_get_list(self):
        # TODO - these should be generated from rc file!
        dicts = ['wkv_variable', 'gcmd_instrument', 'gcmd_science_keyword',
                 'gcmd_provider', 'gcmd_platform', 'gcmd_location',
                 'gcmd_horizontalresolutionrange',
                 'gcmd_verticalresolutionrange',
                 'gcmd_temporalresolutionrange', 'gcmd_project',
                 'gcmd_rucontenttype', 'cf_standard_name',
                 'iso19115_topic_category',
                 'mmd_access_constraints',
                 'mmd_activity_type',
                 'mmd_areas',
                 'mmd_operstatus',
                 'mmd_platform_type',
                 'mmd_use_constraint_type'
                 ]
        for name in dicts:
            function = getattr(pti, 'get_' + name + '_list')
            self.assertIsInstance(function(), list)

    def test_remove_and_get_gcmd_instrument(self):
        if os.path.exists(resource_filename('pythesint', 'json')):
            shutil.rmtree(resource_filename('pythesint', 'json'))
        item = 'MERIS'
        self.assertIsInstance(pti.get_gcmd_instrument(item),
                collections.OrderedDict)

    def test_get_gcmd_instrument(self):
        item = 'MERIS'
        self.assertIsInstance(pti.get_gcmd_instrument(item),
                collections.OrderedDict)

    def test_search_gcmd_instrument_list(self):
        item = 'MERIS'
        self.assertIsInstance(pti.search_gcmd_instrument_list(item)[0],
                collections.OrderedDict)

    def test_get_gcmd_science_keyword(self):
        item = 'sigma naught'
        self.assertIsInstance(pti.get_gcmd_science_keyword(item),
                collections.OrderedDict)

    def test_get_gcmd_provider(self):
        item = 'NERSC'
        self.assertIsInstance(pti.get_gcmd_provider(item),
                collections.OrderedDict)

    def test_get_gcmd_platform(self):
        item = 'AQUA'
        self.assertIsInstance(pti.get_gcmd_platform(item),
                collections.OrderedDict)

    def test_get_gcmd_location(self):
        item = 'NORWEGIAN SEA'
        self.assertIsInstance(pti.get_gcmd_location(item),
                collections.OrderedDict)

    def test_get_gcmd_horizontalresolutionrange(self):
        item = '< 1 meter'
        self.assertIsInstance(pti.get_gcmd_horizontalresolutionrange(item),
                collections.OrderedDict)

    def test_get_gcmd_verticalresolutionrange(self):
        item = '< 1 meter'
        self.assertIsInstance(pti.get_gcmd_verticalresolutionrange(item),
                collections.OrderedDict)

    def test_get_gcmd_temporalresolutionrange(self):
        item = 'Decadal'
        self.assertIsInstance(pti.get_gcmd_temporalresolutionrange(item),
                collections.OrderedDict)

    def test_get_gcmd_project(self):
        item = 'IPCC'
        self.assertIsInstance(pti.get_gcmd_project(item),
                collections.OrderedDict)

    def test_get_gcmd_rucontenttype(self):
        item = 'THREDDS DATA'
        self.assertIsInstance(pti.get_gcmd_rucontenttype(item),
                collections.OrderedDict)

    def test_get_cf_standard_name(self):
        item = 'surface_backwards_scattering_coefficient_of_radar_wave'
        self.assertIsInstance(pti.get_cf_standard_name(item),
                collections.OrderedDict)

    def test_get_wkv_variable(self):
        item = 'surface_backwards_doppler_frequency_shift_of_radar_wave_due_to_surface_velocity'
        self.assertIsInstance(pti.get_wkv_variable(item),
                collections.OrderedDict)

    def test_get_wkv_latitude(self):
        item = 'latitude'
        self.assertIsInstance(pti.get_wkv_variable(item),
                collections.OrderedDict)

    def test_get_iso19115_topic_category(self):
        item = 'Oceans'
        self.assertIsInstance(pti.get_iso19115_topic_category(item),
                collections.OrderedDict)

    def test_get_mmd_access_constraints(self):
        item = 'open'
        self.assertIsInstance(pti.get_mmd_access_constraints(item),
                collections.OrderedDict)

    def test_get_mmd_activity_type(self):
        item = 'Maps/Charts/Photographs'
        self.assertIsInstance(pti.get_mmd_activity_type(item),
                collections.OrderedDict)

    def test_get_mmd_areas(self):
        item = 'Norwegian Sea'
        self.assertIsInstance(pti.get_mmd_areas(item),
                collections.OrderedDict)

    def test_get_mmd_operstatus(self):
        item = 'scientific'
        self.assertIsInstance(pti.get_mmd_operstatus(item),
                collections.OrderedDict)

    def test_get_mmd_platform_type(self):
        item = 'Sentinel-2A'
        self.assertIsInstance(pti.get_mmd_platform_type(item),
                collections.OrderedDict)

    def test_get_mmd_use_constraint_type(self):
        item = 'CC-BY-4.0'
        self.assertIsInstance(pti.get_mmd_use_constraint_type(item),
                collections.OrderedDict)

    def test_get_fake_instrument(self):
        item = 'FakeItem'
        self.assertRaises(IndexError, pti.get_gcmd_instrument, item)

    def test_update_functions_exists(self):
        # TODO - these should be generated from rc file!
        functions = ['update_wkv_variable', 'update_gcmd_instrument',
                     'update_gcmd_science_keyword', 'update_gcmd_provider',
                     'update_gcmd_platform', 'update_gcmd_location',
                     'update_gcmd_horizontalresolutionrange',
                     'update_gcmd_verticalresolutionrange',
                     'update_gcmd_temporalresolutionrange',
                     'update_gcmd_project', 'update_gcmd_rucontenttype',
                     'update_cf_standard_name',
                     'update_iso19115_topic_category',
                     'update_mmd_access_constraints',
                     'update_mmd_activity_type',
                     'update_mmd_areas',
                     'update_mmd_operstatus',
                     'update_mmd_platform_type',
                     'update_mmd_use_constraint_type'
                     ]
        for function in functions:
            self.assertTrue(hasattr(pti, function), 'Function is missing:%s' %
                            (function))

    def test_urls(self):
        # items() in general is inefficient on Python 2, but should be no problem with just a couple items.
        for key, voc in pti.pythesint.vocabularies.items():
            if hasattr(voc, 'url'):
                response = requests.get(voc.url)
                self.assertEqual(response.status_code, 200)

    def test_update_all_mocked(self):
        vocabularies_names = (
            '1',
            'anothervocab',
            'thirdvocab',
            'instruments',
            'something'
        )

        vocabularies = {}
        for name in vocabularies_names:
            vocabulary_mock = MagicMock()
            vocabulary_mock.version = None
            vocabularies[name] = vocabulary_mock

        with patch('pythesint.pythesint.vocabularies', vocabularies):
            pti.update_all_vocabularies()
            # items() in general is inefficient on Python 2, but should be no problem with just a couple items.
            for _, mock in pti.pythesint.vocabularies.items():
                mock.update.assert_called_once_with(version=None)

if __name__ == "__main__":
    unittest.main()



from __future__ import absolute_import

from collections import OrderedDict
import requests
import warnings

from pythesint.json_vocabulary import JSONVocabulary


class GCMDVocabulary(JSONVocabulary):

    def _check_categories(self, categories):
        '''Print a warning if the categories are not the expected ones
        '''
        if set(self.categories) != set(categories):
            mismatch_categories = set(self.categories).difference(set(categories))
            warnings.warn(f'Unknown categories {mismatch_categories} in {self.name}')

    def _fetch_online_data(self, version=None):
        ''' Return list of GCMD standard keywords
            self.url must be set
        '''
        if version:
            params = {'version': version}
        else:
            params = {}

        try:
            r = requests.get(self.url, verify=False, params=params)
            r.raise_for_status()
        except requests.RequestException:
            print("Could not get the vocabulary file at '{}'".format(self.url))
            raise
        rlines = [line for line in r.text.splitlines()]
        gcmd_list = []

        _read_revision(rlines[0], gcmd_list)

        categories = _get_categories(rlines)
        self._check_categories(categories)

        for line in rlines[2:]:
            _read_line(line, gcmd_list, categories)

        return gcmd_list


def _read_revision(line, gcmd_list):
    ''' Reads the line, extracts the Revision into a new dictionary and appends
    it to gcmd_list
    '''
    # TODO: Cast exception if not found?
    if 'Keyword Version' and 'Revision' in line:
        meta = line.split('","')
        gcmd_list.append({
            'Revision': meta[1][10:],
            'Keyword Version': meta[0].split(': ')[1]
        })


def _get_categories(lines):
    '''Get the categories from the lines read from the source'''
    return lines[1].split(',')[:-1]


def _read_line(line, gcmd_list, categories):
    ''' Converts line into dictionary values for elements in the categories
    appends the dictionary to gcmd_list
    '''
    gcmd_keywords = line.split('","')
    gcmd_keywords[0] = gcmd_keywords[0].strip('"')
    if gcmd_keywords[0] == 'NOT APPLICABLE':
        return
    # Remove last item (the ID is not needed)
    gcmd_keywords.pop(-1)
    # skip record if it is longer than the definition of categories
    if len(gcmd_keywords) > len(categories):
        return
    line_kw = OrderedDict()
    for i, key in enumerate(categories):
        if i < len(gcmd_keywords):
            # if the record is equal to definition of categories
            line_kw[key] = gcmd_keywords[i]
        else:
            # if the record is shorter than definition of categories: add empty string
            line_kw[key] = ""
    gcmd_list.append(line_kw)



#-------------------------------------------------------------------------------
# Name:     Python Thesaurus Interface
# Purpose:  Python interface to various metadata vocabularies
#
# Author:   Morten Wergeland Hansen, Anton A. Korosov, Aleksander Vines
# Modified: 07.03.2018
#
# Created:  07.12.2015
# Copyright:(c) NERSC
# License:  GPLv3
#-------------------------------------------------------------------------------
import os
from setuptools import setup, find_packages
from setuptools.command.install_scripts import install_scripts
from os import path

readme_file = 'README.md'
NAME = 'pythesint'
REQS = ['PyYAML<6.0', 'requests', 'xdg<=1.0.7;platform_system!="Windows"']

here = path.abspath(path.dirname(path.realpath(__file__)))

# Get the long description from the README file
long_description = ''
long_description_content_type = 'text/plain'
if path.exists(path.join(here, readme_file)):
    long_description = open(path.join(here, readme_file)).read()
    long_description_content_type='text/markdown'

class update_vocabularies(install_scripts):
    def run(self):
        install_scripts.run(self)
        import pythesint as pti
        pti.update_all_vocabularies()

setup(
    name=NAME,

    version=os.getenv('PYTHESINT_RELEASE', '0.0.0dev').lstrip('v'),

    description='A Python interface to various metadata vocabularies',
    long_description=long_description,
    long_description_content_type=long_description_content_type,

    zip_safe=False,

    author='Morten W. Hansen, Anton A. Korosov, Aleksander Vines',

    author_email='morten.hansen@nersc.no',

    url='https://github.com/nansencenter/py-thesaurus-interface',

    download_url='https://github.com/nansencenter/py-thesaurus-interface/archive/v1.6.5.tar.gz',

    packages=find_packages(),

    include_package_data=True,

    setup_requires=REQS,

    install_requires=REQS,

    test_suite='pythesint.tests',

    license='GPLv3',

    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Plugins',
        'Environment :: Console',
        'Intended Audience :: Information Technology',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Scientific/Engineering',
        'Topic :: Scientific/Engineering :: Atmospheric Science',
        'Topic :: Scientific/Engineering :: Information Analysis',
        'Topic :: Utilities',
    ],

    keywords='metadata standards thesaurus vocabulary',

    cmdclass = {'install_scripts': update_vocabularies}
)



import io
import unittest
import xml.dom.minidom
from collections import OrderedDict

import mock.mock as mock

from pythesint.mmd_vocabulary import MMDVocabulary

class MMDVocabularyTest(unittest.TestCase):

    base_file_contents = '''<?xml version="1.0"?>
        <rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
            xmlns:skos="http://www.w3.org/2004/02/skos/core#"
            xmlns:isothes="http://purl.org/iso25964/skos-thes#">
        <skos:Collection rdf:about="https://vocab.met.no/mmd/concept">
            <rdf:type rdf:resource="http://purl.org/iso25964/skos-thes#ConceptGroup"/>
            <skos:inScheme rdf:resource="https://vocab.met.no/mmd"/>
            <skos:prefLabel xml:lang="en">Concept label</skos:prefLabel>
            <skos:definition xml:lang="en">Concept definition</skos:definition>
            <skos:member>
            <skos:Concept rdf:about="https://vocab.met.no/mmd/concept/instance">
                <skos:prefLabel xml:lang="en">Instance label</skos:prefLabel>
                <skos:definition xml:lang="en">Instance definition</skos:definition>
            </skos:Concept>
            </skos:member>
        </skos:Collection>
        </rdf:RDF>
    '''

    def setUp(self):
        self.voc = MMDVocabulary(name='test_voc', collection_label='Concept label')

    def test_get_subnode_data(self):
        """Test getting the data from a subnode"""
        dom = xml.dom.minidom.parseString(self.base_file_contents)
        collection = dom.childNodes[0].getElementsByTagName('skos:Collection')[0]
        self.assertEqual(
            self.voc.get_subnode_data(collection, 'skos:definition'),
            'Concept definition')

    def test_get_element_by_label(self):
        """Test getting the collection having a particular label"""
        dom = xml.dom.minidom.parseString(self.base_file_contents)
        self.assertEqual(
            self.voc.get_element_by_label(dom.documentElement, 'skos:Collection', 'Concept label'),
            dom.childNodes[0].getElementsByTagName('skos:Collection')[0])


    def test_fetch_online_data(self):
        """Test that right collection is extracted"""
        # We use the exception as side effect to skip the part of the
        # method we are not testing.
        # TODO: split _fetch_online_data() into several methods instead

        expected_list = [
            OrderedDict([
                ('aboutCollection', 'https://vocab.met.no/mmd/concept'),
                ('prefLabelCollection', 'Concept label'),
                ('definitionCollection', 'Concept definition')]),
            OrderedDict([
                ('prefLabel', 'Instance label'),
                ('definition', 'Instance definition')])
        ]

        with mock.patch.object(self.voc, '_download_data_file') as mock_download, \
                mock.patch('pythesint.mmd_vocabulary.open') as mock_open:
            # return a new buffer containing the desired contents
            # every time open is called
            mock_open.side_effect = lambda *a, **k: io.StringIO(self.base_file_contents)

            version = 'master'
            expected_list[0]['version'] = version
            # No version provided
            self.assertCountEqual(self.voc._fetch_online_data(), expected_list)
            mock_download.assert_called_with(version=version)

            self.assertCountEqual(self.voc._fetch_online_data(version=None), expected_list)
            mock_download.assert_called_with(version=version)

            self.assertCountEqual(self.voc._fetch_online_data(version=''), expected_list)
            mock_download.assert_called_with(version=version)

            # Version provided
            version = 'a5c8573'
            expected_list[0]['version'] = version
            self.assertCountEqual(self.voc._fetch_online_data(version=version), expected_list)
            mock_download.assert_called_with(version='a5c8573')

    def test_get_local_file_github_blob_sha(self):
        """Test that the correct hash is returned"""
        buffer = io.BytesIO(b'file contents\n')
        with mock.patch('pythesint.mmd_vocabulary.open') as mock_open, \
                mock.patch('os.path.isfile', return_value=True):
            mock_open.return_value.__enter__.return_value = buffer
            self.assertEqual(
                MMDVocabulary._get_local_file_github_blob_sha('test.txt'),
                'd03e2425cf1c82616e12cb430c69aaa6cc08ff84')

    def test_get_remote_file_github_blob_sha(self):
        """Test that the correct sha is fetched from github's API"""
        response = '''{
            "sha": "20b5eab06834b02b609097e9285dba1e2d266fe0",
            "url": "https://api.github.com/repos/metno/mmd/git/trees/20b5eab06834b02b609097e9285dba1e2d266fe0",
            "tree": [
                {
                "path": "mmd-vocabulary.ttl",
                "mode": "100644",
                "type": "blob",
                "sha": "21bc582a136d0dac8461f66e169f25e067f05e3b",
                "size": 89829,
                "url": "https://api.github.com/repos/metno/mmd/git/blobs/21bc582a136d0dac8461f66e169f25e067f05e3b"
                },
                {
                "path": "mmd-vocabulary.xml",
                "mode": "100644",
                "type": "blob",
                "sha": "4105803f5f54789116ba840afb5b4e7c22cb4286",
                "size": 114697,
                "url": "https://api.github.com/repos/metno/mmd/git/blobs/4105803f5f54789116ba840afb5b4e7c22cb4286"
                }
            ],
            "truncated": false
            }
            '''
        with mock.patch('requests.get') as mock_get:
            mock_get.return_value.text = response
            self.assertEqual(
                self.voc._get_remote_file_github_blob_sha('master'),
                '4105803f5f54789116ba840afb5b4e7c22cb4286')
            mock_get.assert_called_with(
                'https://api.github.com/repos/metno/mmd/git/trees/master:thesauri')


class DownloadTestCase(unittest.TestCase):
    """Tests for the MMDVocabulary._download_data_file() method"""

    def setUp(self):
        self.voc = MMDVocabulary(name='test_voc', collection_label='foo')
        self.mock_local_sha = mock.patch.object(self.voc, '_get_local_file_github_blob_sha').start()
        self.mock_remote_sha = mock.patch.object(
            self.voc, '_get_remote_file_github_blob_sha').start()
        self.mock_open = mock.patch('pythesint.mmd_vocabulary.open').start()
        self.mock_get = mock.patch('requests.get').start()
        self.addCleanup(mock.patch.stopall)

    def test_download_data_file_if_not_present(self):
        """The file should be downloaded if it is not already there"""
        with mock.patch('os.path.isfile', return_value=False):
            self.voc._download_data_file()

        self.mock_get.assert_called_with(self.voc.base_url, stream=True)

    def test_download_data_file_if_sha_does_not_match(self):
        """The file should be downloaded if the sha hashes do not match"""
        with mock.patch('os.path.isfile', return_value=True):
            self.mock_local_sha.return_value = 'a4b5'
            self.mock_remote_sha.return_value = 'b4c2'
            self.voc._download_data_file()

        self.mock_get.assert_called_with(self.voc.base_url, stream=True)

    def test_dont_download_data_file_if_sha_match(self):
        """The file should not be downloaded if the sha hashes match"""
        with mock.patch('os.path.isfile', return_value=True):
            self.mock_local_sha.return_value = 'a4b5'
            self.mock_remote_sha.return_value = 'a4b5'
            self.voc._download_data_file()

        self.mock_get.assert_not_called()

    def test_download_data_file_version(self):
        """Test that the specified version is downloaded"""
        with mock.patch('os.path.isfile', return_value=False):
            self.voc._download_data_file(version='v3.2')

        self.mock_get.assert_called_with(
            'https://raw.githubusercontent.com/metno/mmd/v3.2/thesauri/mmd-vocabulary.xml',
            stream=True)



from __future__ import absolute_import

import yaml
import requests

import pythesint.utils as utils
from pythesint.json_vocabulary import JSONVocabulary


class WKVVocabulary(JSONVocabulary):
    def _fetch_online_data(self, version=None):
        '''Return list of Well Known Variables from Nansat'''
        if version:
            url = utils.set_github_version(self.url, version)
        else:
            url = self.url

        try:
            r = requests.get(url)
        except requests.RequestException:
            print("Could not get the vocabulary file at '{}'".format(self.url))
            raise

        if version:
            wkv_list = [{'version': version}]
        else:
            wkv_list = []
        wkv_list.extend(yaml.safe_load(r.text))

        return wkv_list

